(function(require, loaders, terminal, buttons) {
  "use strict";
  var api = null;

  function loaded() {
    for (var i = 0, length = loaders.length; i < length; ++i) {
      loaders[i].classList.remove('loading');
    }
  }

  function message(msg) {
    terminal.value += msg;
    terminal.value += '\n';
    terminal.scrollTop = terminal.scrollHeight;
  }

  function getLocalData() {
    message('Local data:');
    message(JSON.stringify(api.data, null, '  '));
  }

  function getApiData() {
    message('Getting Remote Data');
    api.request.get('').then(function(data) {
      message('Remote data:');
      message(JSON.stringify(data, null, '  '));
    });
  }

  function checkSync() {
    message('Checking Synchronisation');
    api.request.get('').then(function(data) {
      message('In sync: ' + ((JSON.stringify(data) == JSON.stringify(api.data)) ? 'true' : 'false'));
    });
  }

  function reset() {
    message('Resetting data');
    api.reset().then(function() {
      message('Data reset');
    });
  }

  function addZone() {
    message('Adding zone');
    api.data.zones.push({})
    api.data.zones[api.data.zones.length - 1].loadId().then(function(id) {
      message('Added zone ' + id)
    });
  }

  function bindButtons() {
    var methods = {
      addZone: addZone,
      getLocalData: getLocalData,
      getApiData: getApiData,
      checkSync: checkSync,
      reset: reset
    };

    for (var i = 0, length = buttons.length; i < length; ++i) {
      var button = buttons[i];
      var id = button.id;
      if (methods.hasOwnProperty(id)) {
        button.disabled = false;
        button.onclick = methods[id];
      }
    }
  }


  function dataLoaded(data) {
    message('Initial data:');
    getLocalData();
    bindButtons();
    loaded();
  }

  function failure(error) {
    console.error(error.message);
    message(error.message);
    terminal.classList.add('error');
    loaded();
  }

  function success(core, fetch, origin, vca) {
    message('Loading API data');
    api = new vca.Api();
    api.loadData()
      .then(dataLoaded)
      .catch(failure);
  }

  message('Loading vca-api.js');

  require([
    'bower_components/core.js/client/core.min.js',
    'bower_components/fetch/fetch.js',
    'bower_components/ie-location-origin-polyfill/ie-location-origin-polyfill.js',
    'dist/vca-api.min.js'
  ], success, failure);

}(
  window.require,
  window.document.querySelectorAll('.loading'),
  window.document.getElementById('terminal'),
  window.document.getElementById('buttons').querySelectorAll('button')
))
