import defaults from 'json-schema-defaults';
export default Base => class extends Base {
  constructor({id = null, schema, endpoint, emitter, data, dependency}) {
    super({schema, endpoint, emitter, data});
    if ((this._schema.type === 'object') && data.propertyIsEnumerable('id')) {
      throw new TypeError(`The new child data cannot have an 'id' property. It will be retrieved from the API`);
    } else if ((id !== null) && !Number.isInteger(id)) {
      throw new TypeError('The ID must be an integer');
    } else if ((id !== null) && (id < 0)) {
      throw new TypeError(`The ID cannot be less than zero '${id}'`);
    }
    const def = defaults(this._schema);
    const value = Object.isObject(def) ? Object.assign(def, data) : ((data === undefined) ? def : data);
    const result = this._validator.validate(value, this._schema);
    if (result.errors.length) {
      throw result.errors[0];
    }
    Object.defineProperties(this, {
      _id: {
        get: () => {
          return id;
        }
      }
    });
    if (id === null) {
      const json = JSON.stringify(data);
      const idPromise = (dependency ? dependency.loadId() : Promise.resolve(null))
        .then(() => this._request.post('', json, {'Content-Type': 'application/json'}))
        .then(json => {
          if (!Object.isObject(json) || !Number.isInteger(json.index)) {
            throw new TypeError(`The index returned was not an integer: '${JSON.stringify(json)}'`);
          }
          return json.index;
        })
        .then(index => {
          id = index;
          delete this.idPromise;
          this._endpoint += `/${id}`;
          this.emit('resolved', this, id);
          return id;
        });
      Object.defineProperties(this, {
        idPromise: {
          configurable: true,
          value: idPromise
        }
      });
    } else {
      this._endpoint += `/${id}`;
    }
  }
  loadId() {
    if (this.idPromise) {
      return this.idPromise;
    }
    return Promise.resolve(this._id);
  }
};

//# sourceMappingURL=Identifiable.js.map
