import Endpoint from './Endpoint';
import {Validator, ValidationError} from 'jsonschema';
export {ValidationError};
import defaults from 'json-schema-defaults';
export default class Node extends Endpoint {
  constructor({schema, emitter, endpoint, Data = Object}) {
    super({endpoint, emitter});
    if (!Object.isObject(schema)) {
      throw new TypeError(`Schema must be Object: ${schema}`);
    }
    const types = schema.hasOwnProperty('type') ? [schema.type] : schema.oneOf.map(e => e.type);
    const writable = types.every(type => !(type === 'object' || type === 'array'));
    Object.defineProperties(this, {
      _schema: {
        value: schema
      },
      _validator: {
        value: new Validator()
      },
      _data: {
        writable,
        value: new Data()
      }
    });
    Object.freeze(this._schema);
  }
  assign(object) {
    return Object.assign(this, object);
  }
  reset() {
    return this.assign(defaults(this._schema));
  }
}

//# sourceMappingURL=Node.js.map
