import Sparse from './Sparse';
import FixedIdentifiable from '../identifiable/Fixed';
export default class Factory extends Sparse {
  constructor({schema, emitter, endpoint, data}) {
    super({schema, endpoint, emitter, data});
    const keys = Object.keys(this._childSchema);
    const property = keys[0];
    const expected = 'oneOf';
    if ((property !== expected) || (keys.length !== 1)) {
      throw new TypeError(`The child schema for a 'Factory' node must have only a '${expected}' key`);
    }
  }
  createChild({data, id, dependency}) {
    if (!data || !data.hasOwnProperty('typename')) {
      throw new TypeError(`Must provide a 'typename' key of the template to be created`);
    }
    const chosenSchema = this._childSchema.oneOf.find(
      option => {
        const re = new RegExp(option.properties.typename.pattern);
        return re.test(data.typename);
      }
    );
    if (!chosenSchema) {
      throw new TypeError(`There is no valid template for the '${data.typename}' typename`);
    }
    return new FixedIdentifiable({
      data,
      id,
      dependency,
      schema: chosenSchema,
      emitter: this._emitter,
      endpoint: this._endpoint
    });
  }
}

//# sourceMappingURL=Factory.js.map
