import Dynamic from './Dynamic';
import defaults from 'json-schema-defaults';
export default class Vector extends Dynamic {
  constructor({schema, emitter, endpoint, data}) {
    const items = schema.items;
    if (!Object.isObject(items)) {
      throw new TypeError(`The schema for a 'Vector' node must have a 'items' key`);
    }
    super({schema, endpoint, emitter, childSchema: schema.items});
    const length = this.length;
    const children = data || defaults(this._schema);
    this._data.push(...children.map((data, id) => this.createChild({data, id})));
    this.updateProperties(length);
  }
  toJSON() {
    return this._data;
  }
  splice(start, deleteCount, ...elements) {
    const old = this.length;
    deleteCount = (deleteCount === undefined) ? (old - start) : deleteCount;
    const copy = this._data.slice();
    const removed = copy.splice(start, deleteCount, ...elements).map(o => this.createChild({data: o, id: o._id}));
    let child = null;
    for (let i = this.length, length = copy.length; i < length; ++i) {
      child = this.createChild({data: copy[i], dependency: child});
      this._data.push(child);
    }
    for (let i = 0, length = old; i < length; ++i) {
      this._data[i].assign(copy[i]);
    }
    const added = this._data.slice(old);
    const gone = this._data.splice(copy.length);
    if (added) {
      this.emit('added', this, start, added);
    }
    if (removed) {
      this.emit('erased', this, start, removed);
    }
    for (const child of gone) {
      child.loadId()
        .then(() => {
          return child._request.delete('');
        })
        .then(response => {
          this.emit('gone', this, child._id, child, response);
        }, e => {
          this.emit('reinsert', e, this, child._id, child);
        });
    }
    this.updateProperties(old);
    return removed;
  }
}

//# sourceMappingURL=Vector.js.map
