import Request from './Request';
import Emitter from './Emitter';

export default class Endpoint extends Emitter {
  /**
   * A base class that can be used for API objects that need to store an API endpoint
   * @param {String} endpoint the endpoint for the `Request` class to be created with
   * @param {EventEmitter} [emitter] the emitter to use for generating events
   * @public
   */
  constructor({endpoint, emitter}) {
    super({emitter});
    if (typeof (endpoint) !== 'string') {
      throw new TypeError(`endpoint must be String: ${endpoint}`);
    }
    Object.defineProperty(this, '_request', {
      value: new Request({endpoint})
    });
  }

  /**
   * @property {String} endpoint the endpoint in the API
  */
  get _endpoint() {
    return this._request.endpoint;
  }
  set _endpoint(value) {
    const regex = new RegExp(`^${this._request.endpoint}`);
    this._request.endpoint = value;
    for (const key in this) {
      if (this.hasOwnProperty(key) && Object.isObject(this[key]) && this[key]._endpoint) {
        this[key]._endpoint = this[key]._endpoint.replace(regex, value);
      }
    }
  }
}
