import Request from '../../../lib/api/Request.js';

describe('Request', () => {
  beforeEach(function () {
    this.request = new Request();
  });

  it('can be default constructed', function () {
    this.request.should.be.instanceOf(Request);
  });

  describe('fetch', () => {
    it('can retrieve the middleware version with headers', function () {
      const headers = {
        Accept: 'application/json'
      };
      return this.request.fetch('software/vca/api/middleware/version', {headers})
        .then(response => response.json())
        .should.eventually.have.all.keys('major', 'minor', 'patch', 'commit');
    });

    it('can retrieve the middleware version with extension', function () {
      return this.request.fetch('software/vca/api/middleware/version.json')
        .then(response => response.json())
        .should.eventually.have.all.keys('major', 'minor', 'patch', 'commit');
    });
  });

  describe('send', () => {
    it('can retrieve the middleware version', function () {
      return this.request.send({method: 'GET', path: 'software/vca/api/middleware/version'})
        .should.eventually.have.all.keys('major', 'minor', 'patch', 'commit');
    });
  });

  describe('add', () => {
    it('can add a zone', function () {
      return this.request.add('zones');
    });
  });

  describe('test', () => {
    it('can add and then test an action', function () {
      return this.request.add('actions', {typename: 'tcp'}).then(index => {
        return this.request.post(`test/actions/${index}`, {name: 'test'});
      });
    });
  });

  describe('erase', () => {
    it('can add and then erase a zone', function () {
      return this.request.add('zones').then(index => {
        return this.request.erase(`zones/${index}`);
      });
    });

    it('can add and then erase a observable', function () {
      return this.request.add('observables', {typename: 'cpu'}).then(index => {
        return this.request.erase(`observables/${index}`);
      });
    });
  });
});
